
from odoo import api
from odoo.tests import tagged

from odoo.addons.loempia.tests.common import MockRequest, TestLoempia
from odoo.addons.loempia.util import RECENT_THRESHOLD, DL_HIT_THRESHOLD
from odoo.addons.loempia.controllers.frontend import LoempiaFrontendController

from datetime import date
from dateutil.relativedelta import relativedelta


@tagged('post_install', '-at_install')
class TestLoempiaDownload(TestLoempia):

    @classmethod
    def setUpClass(self):
        super(TestLoempiaDownload, self).setUpClass()
        self.Controller = LoempiaFrontendController()

    def test_aggregate_downloads(self):
        """This test ensures that we properly aggregate downloads older than a certain date (6*TIMETHRESHOLD)"""
        env = api.Environment(self.env.cr, self.manager.id, {})
        Downloads = env['loempia.module.download']
        dl_date = date.today() - relativedelta(days=6*RECENT_THRESHOLD+1)
        vals = {'module_id': self.module_13_A.id, 'template_id': self.module_13_A.template_id.id, 'date': dl_date, 'quantity': 2}

        # Create downloads to aggregate
        Downloads.create(vals)
        Downloads.create(vals)
        vals.update(hits=DL_HIT_THRESHOLD)
        Downloads.create(vals)
        vals.update(hits=1, date=date.today())
        Downloads.create(vals)

        Downloads._aggregate_downloads()

        self.assertEqual(Downloads.search_count([]), 3)

    def test_fail_download(self):
        """
        This test has for purpose to ensure that fraudulent attempt of
        download fails.
        """
        self.module_13_A.toggle_block()

        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            # 01
            response = self.Controller._download(self.module_13_A.name, "FAKE", self.module_13_A.download_hash)
            self.assertEqual(response.code, 404, msg="Request not found")
            self.assertEqual(len(self.module_13_A.download_ids), 0, msg="No existing module for this series")

            # 02
            response = self.Controller._download(self.module_13_A.name, self.module_13_A.series_id.name, "FAKE")
            self.assertEqual(response.code, 404, msg="Request not found")
            self.assertEqual(len(self.module_13_A.download_ids), 0, msg="Wrong download hash for module")

            # 03
            response = self.Controller._download(self.module_13_A.name, self.module_13_A.series_id.name, self.module_13_A.download_hash)
            self.assertEqual(response.status_code, 303, msg="Request not found but different")
            self.assertEqual(len(self.module_13_A.download_ids), 0, msg="User has not purchase")

            # 04
            response = self.Controller._download(self.module_13_A.name, self.module_13_A.series_id.name, self.module_13_A.download_hash, date.today())
            self.assertEqual(response.template, "loempia.expired_link", msg="Hash is invalid")
            self.assertEqual(len(self.module_13_A.download_ids), 0, msg="User has not purchase")
